@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
 
 package Device::BusPirate;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 use Carp;
 
@@ -28,6 +28,11 @@ use Module::Pluggable
    sub_name    => "chips";
 my %CHIPMAP = map { $_->can( "CHIP" ) ? ( $_->CHIP => $_ ) : () } __PACKAGE__->chips;
 
+use Struct::Dumb qw( readonly_struct );
+
+readonly_struct Reader => [qw( length future )];
+readonly_struct Alarm  => [qw( time future )];
+
 use constant BUS_PIRATE => $ENV{BUS_PIRATE} || "/dev/ttyUSB0";
 use constant PIRATE_DEBUG => $ENV{PIRATE_DEBUG};
 
@@ -118,6 +123,8 @@ sub new
 
    return bless {
       fh => $fh,
+      alarms => [],
+      readers => [],
    }, $class;
 }
 
@@ -134,7 +141,7 @@ sub write
    my $self = shift;
    my ( $buf ) = @_;
 
-   printf STDERR "PIRATE >> %v02x\n", $buf if PIRATE_DEBUG;
+   printf STDERR "PIRATE >> %v02x\n", $buf if PIRATE_DEBUG > 1;
 
    $self->{fh}->syswrite( $buf );
 }
@@ -143,11 +150,16 @@ sub write
 sub read
 {
    my $self = shift;
-   my ( $n ) = @_;
+   my ( $n, $name ) = @_;
 
-   push @{ $self->{read_f} }, [ $n, my $f = $self->_new_future ];
+   push @{ $self->{readers} }, Reader( $n, my $f = $self->_new_future );
 
-   return $f;
+   return $f unless defined $name;
+
+   return Future->wait_any(
+      $f,
+      $self->sleep( 0.5 )->then_fail( "Timeout waiting for $name" ),
+   );
 }
 
 =head2 $pirate->sleep( $timeout )->get
@@ -162,12 +174,15 @@ sub sleep
    my $self = shift;
    my ( $timeout ) = @_;
 
+   my $alarms = $self->{alarms};
+
    my $until = time() + $timeout;
 
-   croak "Cannot sleep less than existing timeout" if
-      $self->{alarms}[0] and $until < $self->{alarms}[0][0];
+   my $pos = 0;
+   $pos++ while $pos < @$alarms and $alarms->[$pos]->time < $until;
+
+   splice @$alarms, $pos, 0, my $alarm = Alarm( $until, my $f = $self->_new_future );
 
-   push @{ $self->{alarms} }, my $alarm = [ $until, my $f = $self->_new_future ];
    $f->on_cancel( sub {
       $self->{alarms} = [ grep { $_ != $alarm } @{ $self->{alarms} } ];
    });
@@ -288,7 +303,7 @@ sub start
    my $self = shift;
 
    Future->wait_any(
-      $self->read( 5 )->then( sub {
+      $self->read( 5, "start" )->then( sub {
          my ( $buf ) = @_;
          return Future->done( ( $self->{version} ) = $buf =~ m/^BBIO(\d)/ );
       }),
@@ -296,6 +311,7 @@ sub start
          $self->write( "\0" );
          $self->sleep( 0.05 );
       } foreach => [ 1 .. 20 ],
+        while => sub { not shift->failure },
         otherwise => sub {
            Future->fail( "Timed out waiting for device to enter bitbang mode" )
         },
@@ -342,33 +358,33 @@ sub await
 {
    my $bp = shift->{bp};
 
-   my $alarm = $bp->{alarms}[0];
-   my $read = $bp->{read_f}[0];
+   my $alarm  = $bp->{alarms}[0];
+   my $reader = $bp->{readers}[0];
 
    my $fh = $bp->{fh};
 
-   croak "Cannot await with nothing to do" unless $alarm or $read;
+   croak "Cannot await with nothing to do" unless $alarm or $reader;
 
    my $buf = '';
 
    while(1) {
-      if( $read and length $buf >= $read->[0] ) {
-         printf STDERR "PIRATE << %v02x\n", $buf if Device::BusPirate::PIRATE_DEBUG;
-         shift @{ $bp->{read_f} };
-         $read->[1]->done( substr $buf, 0, $read->[0], "" );
+      if( $reader and length $buf >= $reader->length ) {
+         printf STDERR "PIRATE << %v02x\n", $buf if Device::BusPirate::PIRATE_DEBUG > 1;
+         shift @{ $bp->{readers} };
+         $reader->future->done( substr $buf, 0, $reader->length, "" );
          return;
       }
 
-      my $timeout = $alarm ? $alarm->[0] - time() : undef;
+      my $timeout = $alarm ? $alarm->time - time() : undef;
       my $rvec = '';
-      vec( $rvec, $fh->fileno, 1 ) = 1 if $read;
+      vec( $rvec, $fh->fileno, 1 ) = 1 if $reader;
 
       if( select( $rvec, undef, undef, $timeout ) ) {
-         $fh->sysread( $buf, $read->[0] - length $buf, length $buf );
+         $fh->sysread( $buf, $reader->length - length $buf, length $buf );
       }
       elsif( $timeout ) {
          shift @{ $bp->{alarms} };
-         $alarm->[1]->done;
+         $alarm->future->done;
          return;
       }
    }
