@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
 
 package Device::BusPirate::Mode;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 use constant {
    CONF_CS     => 0x01,
@@ -68,7 +68,7 @@ sub _start_mode_and_await
    my $pirate = $self->pirate;
 
    $pirate->write( $send );
-   $pirate->read( length $await )->then( sub {
+   $pirate->read( length $await, "start mode" )->then( sub {
       my ( $buf ) = @_;
       return Future->done( $buf ) if $buf eq $await;
       return Future->fail( "Expected '$await' response but got '$buf'" );
@@ -138,7 +138,7 @@ sub _update_peripherals
       ( $self->{aux}    ? CONF_AUX    : 0 ) |
       ( $self->{cs}     ? CONF_CS     : 0 ) )
    );
-   $self->pirate->read( 1 )
+   $self->pirate->read( 1, "update peripherals" )
       ->then( sub {
          my( $buf ) = @_;
          $buf eq "\x01" or return Future->fail( "Expected ACK response to _update_peripherals" );
