@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Device::BusPirate::Mode );
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 use Carp;
 
@@ -17,6 +17,8 @@ use Future::Utils qw( repeat );
 
 use constant MODE => "I2C";
 
+use constant PIRATE_DEBUG => $ENV{PIRATE_DEBUG};
+
 =head1 NAME
 
 C<Device::BusPirate::Mode::I2C> - use C<Device::BusPirate> in I2C mode
@@ -67,9 +69,10 @@ sub start
    my $self = shift;
 
    $self->_start_mode_and_await( "\x02", "I2C" )->then( sub {
-      $self->pirate->read( 1 )
+      $self->pirate->read( 1, "I2C start" )
    })->then( sub {
       ( $self->{version} ) = @_;
+      print STDERR "PIRATE I2C STARTED\n" if PIRATE_DEBUG;
       return Future->done( $self );
    });
 }
@@ -84,8 +87,10 @@ sub start_bit
 {
    my $self = shift;
 
+   print STDERR "PIRATE I2C START-BIT\n" if PIRATE_DEBUG;
+
    $self->pirate->write( "\x02" );
-   $self->pirate->read( 1 )->then( $EXPECT_ACK )
+   $self->pirate->read( 1, "I2C start_bit" )->then( $EXPECT_ACK )
       ->else_fail( "Expected ACK response to I2C start_bit" );
 }
 
@@ -99,8 +104,10 @@ sub stop_bit
 {
    my $self = shift;
 
+   print STDERR "PIRATE I2C STOP-BIT\n" if PIRATE_DEBUG;
+
    $self->pirate->write( "\x03" );
-   $self->pirate->read( 1 )->then( $EXPECT_ACK )
+   $self->pirate->read( 1, "I2C stop_bit" )->then( $EXPECT_ACK )
       ->else_fail( "Expected ACK response to I2C stop_bit" );
 }
 
@@ -117,6 +124,7 @@ sub write
    my $self = shift;
    my ( $bytes ) = @_;
 
+   printf STDERR "PIRATE I2C WRITE %v02X\n", $bytes if PIRATE_DEBUG;
    my @chunks = $bytes =~ m/(.{1,16})/gs;
 
    repeat {
@@ -126,11 +134,11 @@ sub write
 
       $self->pirate->write( chr( 0x10 | $len_1 ) . $bytes );
 
-      $self->pirate->read( 1 )->then( sub {
+      $self->pirate->read( 1, "I2C write await ACK" )->then( sub {
          my ( $buf ) = @_;
          $buf eq "\x01" or return Future->fail( "Expected ACK response during I2C write" );
 
-         $self->pirate->read( length $bytes )
+         $self->pirate->read( length $bytes, "I2C write data" )
       })->then( sub {
          my ( $buf ) = @_;
          $buf =~ m/^\x00+/;
@@ -156,17 +164,24 @@ sub read
 
    my $ret = "";
 
+   print STDERR "PIRATE I2C READING $length\n" if PIRATE_DEBUG;
+
    repeat {
       my $ack = shift;
       $self->pirate->write( "\x04" );
-      $self->pirate->read( 1 )->then( sub {
+
+      $self->pirate->read( 1, "I2C read data" )->then( sub {
          $ret .= $_[0];
          $self->pirate->write( $ack ? "\x06" : "\x07" );
-         $self->pirate->read( 1 )->then( $EXPECT_ACK )
+         $self->pirate->read( 1, "I2C read send ACK" )->then( $EXPECT_ACK )
             ->else_fail( "Expected ACK response to I2C ack_bit" );
       });
    } foreach => [ (1) x ($length-1), 0 ],
-     otherwise => sub { Future->done( $ret ) };
+     while => sub { not shift->failure },
+     otherwise => sub {
+        printf STDERR "PIRATE I2C READ %v02X\n", $ret if PIRATE_DEBUG;
+        Future->done( $ret );
+     };
 }
 
 =head2 $i2c->send( $address, $bytes )->get
