@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
 
 package Device::BusPirate::Mode::SPI;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Device::BusPirate::Mode );
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 use Carp;
 
@@ -81,7 +81,7 @@ sub start
    $self->{speed}   = 0;
 
    $self->_start_mode_and_await( "\x01", "SPI" )->then( sub {
-      $self->pirate->read( 1 )->then( sub {
+      $self->pirate->read( 1, "SPI start" )->then( sub {
          ( $self->{version} ) = @_;
          return Future->done( $self );
       })
@@ -194,7 +194,7 @@ sub configure
    );
    $self->pirate->write( chr( 0x60 | $self->{speed} ) );
 
-   $self->pirate->read( 2 )->then( $EXPECT_ACK )
+   $self->pirate->read( 2, "SPI configure" )->then( $EXPECT_ACK )
       ->else_fail( "Expected ACK response to SPI configure" );
 }
 
@@ -212,7 +212,7 @@ sub chip_select
    $self->{cs} = !!shift;
 
    $self->pirate->write( $self->{cs} ? "\x03" : "\x02" );
-   $self->pirate->read( 1 )->then( $EXPECT_ACK )
+   $self->pirate->read( 1, "SPI chip_select" )->then( $EXPECT_ACK )
       ->else_fail( "Expected ACK response to SPI chip_select" );
 }
 
@@ -249,20 +249,16 @@ sub _writeread
 
       $self->pirate->write( chr( 0x10 | $len_1 ) . $bytes );
 
-      Future->wait_any(
-         $self->pirate->sleep( 0.5 )->then_fail( "Timed out receiving SPI" ),
+      $self->pirate->read( 1, "SPI await ACK" )->then( sub {
+         my ( $buf ) = @_;
+         $buf eq "\x01" or return Future->fail( "Expected ACK response during SPI writeread" );
 
-         $self->pirate->read( 1 )->then( sub {
-            my ( $buf ) = @_;
-            $buf eq "\x01" or return Future->fail( "Expected ACK response during SPI writeread" );
-
-            $self->pirate->read( length $bytes )
-               ->then( sub {
-                  $ret .= $_[0];
-                  Future->done
-               });
-         }),
-      )
+         $self->pirate->read( length $bytes, "SPI read data" )
+            ->then( sub {
+               $ret .= $_[0];
+               Future->done
+            });
+      });
    } foreach => \@chunks,
      while => sub { not shift->failure },
      otherwise => sub { Future->done( $ret ) };
